import type { PageServerLoad } from "./$types";
import { createGame, getMetro, getStops, getTram, type GameData, type GameOptions } from "$lib";
import { error } from "@sveltejs/kit";

export const load: PageServerLoad = async ({ fetch, url, platform }) => {
	const kv = platform?.env?.TCL_GUESSR_KV;
	if (!kv) return error(500, "could not connect to kv");

	const stops = await getStops(fetch);

	const mode = url.searchParams.get("mode");
	if (mode !== "easy" && mode !== "hard" && mode !== "extreme demon ultra miguel") {
		return error(400, "gamemode is invalid");
	}

	const options: GameOptions = {
		mode,
		metro: url.searchParams.has("metro"),
		tram: url.searchParams.has("tram"),
	};

	const lineColors: [GeoJSON.Feature, string][] = [];

	if (options.metro) lineColors.push(...(await getMetro(fetch)));
	if (options.tram) lineColors.push(...(await getTram(fetch)));

	const lineCodes = new Set<string>(lineColors.map(([f, _]) => f.properties!.code_ligne));

	const crossingStops = stops.filter((f) => {
		if (f.properties?.desserte) {
			return f.properties.desserte
				.split(",")
				.map((d: string) => d.split(":")[0])
				.some((d: string) => lineCodes.has(d));
		} else {
			return false;
		}
	});

	const randomStop = crossingStops[Math.floor(Math.random() * crossingStops.length)];

	if (!randomStop) {
		return error(400, "could not select random stop");
	}

	const gameId = await createGame(randomStop, kv);
	const gameData: GameData = {
		lines: options.mode === "easy" || options.mode === "hard" ? lineColors : [],
		stops: options.mode === "easy" ? crossingStops : [],
		stopName: randomStop.properties!.nom,
		gameId,
	};

	return { gameData };
};
